@@ -5,19 +5,29 @@
 
     The reStructuredText domain.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+from typing import TYPE_CHECKING
+
+from six import iteritems
 
 from sphinx import addnodes
+from sphinx.directives import ObjectDescription
 from sphinx.domains import Domain, ObjType
 from sphinx.locale import l_, _
-from sphinx.directives import ObjectDescription
 from sphinx.roles import XRefRole
 from sphinx.util.nodes import make_refnode
 
+if TYPE_CHECKING:
+    from typing import Any, Dict, Iterator, List, Tuple  # NOQA
+    from docutils import nodes  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.builders import Builder  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+
 
 dir_sig_re = re.compile(r'\.\. (.+?)::(.*)$')
 
@@ -28,7 +38,8 @@ class ReSTMarkup(ObjectDescription):
     """
 
     def add_target_and_index(self, name, sig, signode):
-        targetname = name + '-' + self.objtype
+        # type: (unicode, unicode, addnodes.desc_signature) -> None
+        targetname = self.objtype + '-' + name
         if targetname not in self.state.document.ids:
             signode['names'].append(targetname)
             signode['ids'].append(targetname)
@@ -38,19 +49,18 @@ class ReSTMarkup(ObjectDescription):
             objects = self.env.domaindata['rst']['objects']
             key = (self.objtype, name)
             if key in objects:
-                self.env.warn(self.env.docname,
-                              'duplicate description of %s %s, ' %
-                              (self.objtype, name) +
-                              'other instance in ' +
-                              self.env.doc2path(objects[key]),
-                              self.lineno)
+                self.state_machine.reporter.warning(
+                    'duplicate description of %s %s, ' % (self.objtype, name) +
+                    'other instance in ' + self.env.doc2path(objects[key]),
+                    line=self.lineno)
             objects[key] = self.env.docname
         indextext = self.get_index_text(self.objtype, name)
         if indextext:
             self.indexnode['entries'].append(('single', indextext,
-                                              targetname, targetname))
+                                              targetname, '', None))
 
     def get_index_text(self, objectname, name):
+        # type: (unicode, unicode) -> unicode
         if self.objtype == 'directive':
             return _('%s (directive)') % name
         elif self.objtype == 'role':
@@ -59,15 +69,17 @@ class ReSTMarkup(ObjectDescription):
 
 
 def parse_directive(d):
-    """
-    Parses a directive signature. Returns (directive, arguments) string tuple.
-    if no arguments are given, returns (directive, '').
+    # type: (unicode) -> Tuple[unicode, unicode]
+    """Parse a directive signature.
+
+    Returns (directive, arguments) string tuple.  If no arguments are given,
+    returns (directive, '').
     """
     dir = d.strip()
     if not dir.startswith('.'):
         # Assume it is a directive without syntax
         return (dir, '')
-    m = dir_sig_re.match(dir)
+    m = dir_sig_re.match(dir)  # type: ignore
     if not m:
         return (dir, '')
     parsed_dir, parsed_args = m.groups()
@@ -79,6 +91,7 @@ class ReSTDirective(ReSTMarkup):
     Description of a reST directive.
     """
     def handle_signature(self, sig, signode):
+        # type: (unicode, addnodes.desc_signature) -> unicode
         name, args = parse_directive(sig)
         desc_name = '.. %s::' % name
         signode += addnodes.desc_name(desc_name, desc_name)
@@ -92,6 +105,7 @@ class ReSTRole(ReSTMarkup):
     Description of a reST role.
     """
     def handle_signature(self, sig, signode):
+        # type: (unicode, addnodes.desc_signature) -> unicode
         signode += addnodes.desc_name(':%s:' % sig, ':%s:' % sig)
         return sig
 
@@ -115,23 +129,59 @@ class ReSTDomain(Domain):
     }
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
-    }
+    }  # type: Dict[unicode, Dict[unicode, Tuple[unicode, ObjType]]]
 
     def clear_doc(self, docname):
-        for (typ, name), doc in self.data['objects'].items():
+        # type: (unicode) -> None
+        for (typ, name), doc in list(self.data['objects'].items()):
             if doc == docname:
                 del self.data['objects'][typ, name]
 
+    def merge_domaindata(self, docnames, otherdata):
+        # type: (List[unicode], Dict) -> None
+        # XXX check duplicates
+        for (typ, name), doc in otherdata['objects'].items():
+            if doc in docnames:
+                self.data['objects'][typ, name] = doc
+
     def resolve_xref(self, env, fromdocname, builder, typ, target, node,
                      contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         objects = self.data['objects']
         objtypes = self.objtypes_for_role(typ)
         for objtype in objtypes:
             if (objtype, target) in objects:
                 return make_refnode(builder, fromdocname,
                                     objects[objtype, target],
-                                    target, contnode, target)
+                                    objtype + '-' + target,
+                                    contnode, target + ' ' + objtype)
+
+    def resolve_any_xref(self, env, fromdocname, builder, target,
+                         node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, nodes.Node, nodes.Node) -> List[nodes.Node]  # NOQA
+        objects = self.data['objects']
+        results = []
+        for objtype in self.object_types:
+            if (objtype, target) in self.data['objects']:
+                results.append(('rst:' + self.role_for_objtype(objtype),
+                                make_refnode(builder, fromdocname,
+                                             objects[objtype, target],
+                                             objtype + '-' + target,
+                                             contnode, target + ' ' + objtype)))
+        return results
 
     def get_objects(self):
-        for (typ, name), docname in self.data['objects'].iteritems():
-            yield name, name, typ, docname, name, 1
+        # type: () -> Iterator[Tuple[unicode, unicode, unicode, unicode, unicode, int]]
+        for (typ, name), docname in iteritems(self.data['objects']):
+            yield name, name, typ, docname, typ + '-' + name, 1
+
+
+def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
+    app.add_domain(ReSTDomain)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
