@@ -14,72 +14,86 @@
        generate:
                sphinx-autogen -o source/generated source/*.rst
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import print_function
 
+import argparse
+import codecs
 import os
+import pydoc
 import re
 import sys
-import pydoc
-import optparse
+from typing import TYPE_CHECKING
 
 from jinja2 import FileSystemLoader, TemplateNotFound
 from jinja2.sandbox import SandboxedEnvironment
 
+from sphinx import __display_version__
 from sphinx import package_dir
 from sphinx.ext.autosummary import import_by_name, get_documenter
 from sphinx.jinja2glue import BuiltinTemplateLoader
-from sphinx.util.osutil import ensuredir
+from sphinx.registry import SphinxComponentRegistry
 from sphinx.util.inspect import safe_getattr
+from sphinx.util.osutil import ensuredir
+from sphinx.util.rst import escape as rst_escape
 
-# Add documenters to AutoDirective registry
-from sphinx.ext.autodoc import add_documenter, \
-    ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter, \
-    FunctionDocumenter, MethodDocumenter, AttributeDocumenter, \
-    InstanceAttributeDocumenter
-add_documenter(ModuleDocumenter)
-add_documenter(ClassDocumenter)
-add_documenter(ExceptionDocumenter)
-add_documenter(DataDocumenter)
-add_documenter(FunctionDocumenter)
-add_documenter(MethodDocumenter)
-add_documenter(AttributeDocumenter)
-add_documenter(InstanceAttributeDocumenter)
+if TYPE_CHECKING:
+    from typing import Any, Callable, Dict, Tuple, List  # NOQA
+    from jinja2 import BaseLoader  # NOQA
+    from sphinx import addnodes  # NOQA
+    from sphinx.builders import Builder  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
 
 
-def main(argv=sys.argv):
-    usage = """%prog [OPTIONS] SOURCEFILE ..."""
-    p = optparse.OptionParser(usage.strip())
-    p.add_option("-o", "--output-dir", action="store", type="string",
-                 dest="output_dir", default=None,
-                 help="Directory to place all output in")
-    p.add_option("-s", "--suffix", action="store", type="string",
-                 dest="suffix", default="rst",
-                 help="Default suffix for files (default: %default)")
-    p.add_option("-t", "--templates", action="store", type="string",
-                 dest="templates", default=None,
-                 help="Custom template directory (default: %default)")
-    options, args = p.parse_args(argv[1:])
+class DummyApplication(object):
+    """Dummy Application class for sphinx-autogen command."""
 
-    if len(args) < 1:
-        p.error('no input files given')
+    def __init__(self):
+        # type: () -> None
+        self.registry = SphinxComponentRegistry()
+
+
+def setup_documenters(app):
+    # type: (Any) -> None
+    from sphinx.ext.autodoc import (
+        ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter,
+        FunctionDocumenter, MethodDocumenter, AttributeDocumenter,
+        InstanceAttributeDocumenter
+    )
+    documenters = [
+        ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter,
+        FunctionDocumenter, MethodDocumenter, AttributeDocumenter,
+        InstanceAttributeDocumenter
+    ]
+    for documenter in documenters:
+        app.registry.add_documenter(documenter.objtype, documenter)
 
-    generate_autosummary_docs(args, options.output_dir,
-                              "." + options.suffix,
-                              template_dir=options.templates)
 
 def _simple_info(msg):
-    print msg
+    # type: (unicode) -> None
+    print(msg)
+
 
 def _simple_warn(msg):
-    print >> sys.stderr, 'WARNING: ' + msg
+    # type: (unicode) -> None
+    print('WARNING: ' + msg, file=sys.stderr)
+
+
+def _underline(title, line='='):
+    if '\n' in title:
+        raise ValueError('Can only underline single lines')
+    return title + '\n' + line * len(title)
+
 
 # -- Generating output ---------------------------------------------------------
 
 def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
                               warn=_simple_warn, info=_simple_info,
-                              base_path=None, builder=None, template_dir=None):
+                              base_path=None, builder=None, template_dir=None,
+                              imported_members=False, app=None):
+    # type: (List[unicode], unicode, unicode, Callable, Callable, unicode, Builder, unicode, bool, Any) -> None  # NOQA
 
     showed_sources = list(sorted(sources))
     if len(showed_sources) > 20:
@@ -94,8 +108,11 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
         sources = [os.path.join(base_path, filename) for filename in sources]
 
     # create our own templating environment
+    template_dirs = None  # type: List[unicode]
     template_dirs = [os.path.join(package_dir, 'ext',
                                   'autosummary', 'templates')]
+
+    template_loader = None  # type: BaseLoader
     if builder is not None:
         # allow the user to override the templates
         template_loader = BuiltinTemplateLoader()
@@ -103,20 +120,22 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
     else:
         if template_dir:
             template_dirs.insert(0, template_dir)
-        template_loader = FileSystemLoader(template_dirs)
+        template_loader = FileSystemLoader(template_dirs)  # type: ignore
     template_env = SandboxedEnvironment(loader=template_loader)
+    template_env.filters['underline'] = _underline
+
+    # replace the builtin html filters
+    template_env.filters['escape'] = rst_escape
+    template_env.filters['e'] = rst_escape
 
     # read
     items = find_autosummary_in_files(sources)
 
-    # remove possible duplicates
-    items = dict([(item, True) for item in items]).keys()
-
     # keep track of new files
     new_files = []
 
     # write
-    for name, path, template_name in sorted(items, key=str):
+    for name, path, template_name in sorted(set(items), key=str):
         if path is None:
             # The corresponding autosummary:: directive did not have
             # a :toctree: option
@@ -126,8 +145,8 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
         ensuredir(path)
 
         try:
-            name, obj, parent = import_by_name(name)
-        except ImportError, e:
+            name, obj, parent, mod_name = import_by_name(name)
+        except ImportError as e:
             warn('[autosummary] failed to import %r: %s' % (name, e))
             continue
 
@@ -139,10 +158,8 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
 
         new_files.append(fn)
 
-        f = open(fn, 'w')
-
-        try:
-            doc = get_documenter(obj, parent)
+        with open(fn, 'w') as f:
+            doc = get_documenter(app, obj, parent)
 
             if template_name is not None:
                 template = template_env.get_template(template_name)
@@ -153,36 +170,39 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
                 except TemplateNotFound:
                     template = template_env.get_template('autosummary/base.rst')
 
-            def get_members(obj, typ, include_public=[]):
-                items = []
+            def get_members(obj, typ, include_public=[], imported=True):
+                # type: (Any, unicode, List[unicode], bool) -> Tuple[List[unicode], List[unicode]]  # NOQA
+                items = []  # type: List[unicode]
                 for name in dir(obj):
                     try:
-                        documenter = get_documenter(safe_getattr(obj, name),
-                                                    obj)
+                        value = safe_getattr(obj, name)
                     except AttributeError:
                         continue
+                    documenter = get_documenter(app, value, obj)
                     if documenter.objtype == typ:
-                        items.append(name)
+                        if imported or getattr(value, '__module__', None) == obj.__name__:
+                            # skip imported members if expected
+                            items.append(name)
                 public = [x for x in items
                           if x in include_public or not x.startswith('_')]
                 return public, items
 
-            ns = {}
+            ns = {}  # type: Dict[unicode, Any]
 
             if doc.objtype == 'module':
                 ns['members'] = dir(obj)
                 ns['functions'], ns['all_functions'] = \
-                                   get_members(obj, 'function')
+                    get_members(obj, 'function', imported=imported_members)
                 ns['classes'], ns['all_classes'] = \
-                                 get_members(obj, 'class')
+                    get_members(obj, 'class', imported=imported_members)
                 ns['exceptions'], ns['all_exceptions'] = \
-                                   get_members(obj, 'exception')
+                    get_members(obj, 'exception', imported=imported_members)
             elif doc.objtype == 'class':
                 ns['members'] = dir(obj)
                 ns['methods'], ns['all_methods'] = \
-                                 get_members(obj, 'method', ['__init__'])
+                    get_members(obj, 'method', ['__init__'])
                 ns['attributes'], ns['all_attributes'] = \
-                                 get_members(obj, 'attribute')
+                    get_members(obj, 'attribute')
 
             parts = name.split('.')
             if doc.objtype in ('method', 'attribute'):
@@ -202,49 +222,55 @@ def generate_autosummary_docs(sources, output_dir=None, suffix='.rst',
             ns['underline'] = len(name) * '='
 
             rendered = template.render(**ns)
-            f.write(rendered)
-        finally:
-            f.close()
+            f.write(rendered)  # type: ignore
 
     # descend recursively to new files
     if new_files:
         generate_autosummary_docs(new_files, output_dir=output_dir,
                                   suffix=suffix, warn=warn, info=info,
                                   base_path=base_path, builder=builder,
-                                  template_dir=template_dir)
+                                  template_dir=template_dir, app=app)
 
 
 # -- Finding documented entries in files ---------------------------------------
 
 def find_autosummary_in_files(filenames):
+    # type: (List[unicode]) -> List[Tuple[unicode, unicode, unicode]]
     """Find out what items are documented in source/*.rst.
 
     See `find_autosummary_in_lines`.
     """
-    documented = []
+    documented = []  # type: List[Tuple[unicode, unicode, unicode]]
     for filename in filenames:
-        f = open(filename, 'r')
-        lines = f.read().splitlines()
-        documented.extend(find_autosummary_in_lines(lines, filename=filename))
-        f.close()
+        with codecs.open(filename, 'r', encoding='utf-8',  # type: ignore
+                         errors='ignore') as f:
+            lines = f.read().splitlines()
+            documented.extend(find_autosummary_in_lines(lines, filename=filename))
     return documented
 
+
 def find_autosummary_in_docstring(name, module=None, filename=None):
+    # type: (unicode, Any, unicode) -> List[Tuple[unicode, unicode, unicode]]
     """Find out what items are documented in the given object's docstring.
 
     See `find_autosummary_in_lines`.
     """
     try:
-        real_name, obj, parent = import_by_name(name)
+        real_name, obj, parent, modname = import_by_name(name)
         lines = pydoc.getdoc(obj).splitlines()
-        return find_autosummary_in_lines(lines, module=name, filename=filename)
+        return find_autosummary_in_lines(lines, module=name, filename=filename)  # type: ignore
     except AttributeError:
         pass
-    except ImportError, e:
-        print "Failed to import '%s': %s" % (name, e)
+    except ImportError as e:
+        print("Failed to import '%s': %s" % (name, e))
+    except SystemExit as e:
+        print("Failed to import '%s'; the module executes module level "
+              "statement and it might call sys.exit()." % name)
     return []
 
+
 def find_autosummary_in_lines(lines, module=None, filename=None):
+    # type: (List[unicode], Any, unicode) -> List[Tuple[unicode, unicode, unicode]]
     """Find out what items appear in autosummary:: directives in the
     given lines.
 
@@ -264,9 +290,9 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
     toctree_arg_re = re.compile(r'^\s+:toctree:\s*(.*?)\s*$')
     template_arg_re = re.compile(r'^\s+:template:\s*(.*?)\s*$')
 
-    documented = []
+    documented = []  # type: List[Tuple[unicode, unicode, unicode]]
 
-    toctree = None
+    toctree = None  # type: unicode
     template = None
     current_module = module
     in_autosummary = False
@@ -274,7 +300,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
 
     for line in lines:
         if in_autosummary:
-            m = toctree_arg_re.match(line)
+            m = toctree_arg_re.match(line)  # type: ignore
             if m:
                 toctree = m.group(1)
                 if filename:
@@ -282,21 +308,21 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
                                            toctree)
                 continue
 
-            m = template_arg_re.match(line)
+            m = template_arg_re.match(line)  # type: ignore
             if m:
                 template = m.group(1).strip()
                 continue
 
             if line.strip().startswith(':'):
-                continue # skip options
+                continue  # skip options
 
-            m = autosummary_item_re.match(line)
+            m = autosummary_item_re.match(line)  # type: ignore
             if m:
                 name = m.group(1).strip()
                 if name.startswith('~'):
                     name = name[1:]
                 if current_module and \
-                       not name.startswith(current_module + '.'):
+                   not name.startswith(current_module + '.'):
                     name = "%s.%s" % (current_module, name)
                 documented.append((name, toctree, template))
                 continue
@@ -306,7 +332,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
 
             in_autosummary = False
 
-        m = autosummary_re.match(line)
+        m = autosummary_re.match(line)  # type: ignore
         if m:
             in_autosummary = True
             base_indent = m.group(1)
@@ -314,7 +340,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
             template = None
             continue
 
-        m = automodule_re.search(line)
+        m = automodule_re.search(line)  # type: ignore
         if m:
             current_module = m.group(1).strip()
             # recurse into the automodule docstring
@@ -322,7 +348,7 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
                 current_module, filename=filename))
             continue
 
-        m = module_re.match(line)
+        m = module_re.match(line)  # type: ignore
         if m:
             current_module = m.group(2)
             continue
@@ -330,5 +356,60 @@ def find_autosummary_in_lines(lines, module=None, filename=None):
     return documented
 
 
+def get_parser():
+    # type: () -> argparse.ArgumentParser
+    parser = argparse.ArgumentParser(
+        usage='%(prog)s [OPTIONS] <SOURCE_FILE>...',
+        epilog='For more information, visit <http://sphinx-doc.org/>.',
+        description="""
+Generate ReStructuredText using autosummary directives.
+
+sphinx-autogen is a frontend to sphinx.ext.autosummary.generate. It generates
+the reStructuredText files from the autosummary directives contained in the
+given input files.
+
+The format of the autosummary directive is documented in the
+``sphinx.ext.autosummary`` Python module and can be read using::
+
+  pydoc sphinx.ext.autosummary
+""")
+
+    parser.add_argument('--version', action='version', dest='show_version',
+                        version='%%(prog)s %s' % __display_version__)
+
+    parser.add_argument('source_file', nargs='+',
+                        help='source files to generate rST files for')
+
+    parser.add_argument('-o', '--output-dir', action='store',
+                        dest='output_dir',
+                        help='directory to place all output in')
+    parser.add_argument('-s', '--suffix', action='store', dest='suffix',
+                        default='rst',
+                        help='default suffix for files (default: '
+                              '%(default)s)')
+    parser.add_argument('-t', '--templates', action='store', dest='templates',
+                        default=None,
+                        help='custom template directory (default: '
+                              '%(default)s)')
+    parser.add_argument('-i', '--imported-members', action='store_true',
+                        dest='imported_members', default=False,
+                        help='document imported members (default: '
+                              '%(default)s)')
+
+    return parser
+
+
+def main(argv=sys.argv[1:]):
+    # type: (List[str]) -> None
+    app = DummyApplication()
+    setup_documenters(app)
+    args = get_parser().parse_args(argv)
+    generate_autosummary_docs(args.source_file, args.output_dir,
+                              '.' + args.suffix,
+                              template_dir=args.templates,
+                              imported_members=args.imported_members,
+                              app=app)
+
+
 if __name__ == '__main__':
     main()
