@@ -5,7 +5,7 @@
 
     Test the build process with LaTeX builder with the test root.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import print_function
@@ -13,23 +13,24 @@ from __future__ import print_function
 import os
 import re
 from itertools import product
+from shutil import copyfile
 from subprocess import Popen, PIPE
 
-from six import PY3
 import pytest
+from six import PY3
+from test_build_html import ENV_WARNINGS
 
 from sphinx.errors import SphinxError
+from sphinx.testing.util import remove_unicode_literals, strip_escseq
+from sphinx.util import docutils
 from sphinx.util.osutil import cd, ensuredir
 from sphinx.writers.latex import LaTeXTranslator
 
-from util import SkipTest, remove_unicode_literals, strip_escseq, skip_if
-from test_build_html import ENV_WARNINGS
-
 
 LATEX_ENGINES = ['pdflatex', 'lualatex', 'xelatex']
 DOCCLASSES = ['howto', 'manual']
 STYLEFILES = ['article.cls', 'fancyhdr.sty', 'titlesec.sty', 'amsmath.sty',
-              'framed.sty', 'color.sty', 'fancyvrb.sty', 'threeparttable.sty',
+              'framed.sty', 'color.sty', 'fancyvrb.sty',
               'fncychap.sty', 'geometry.sty', 'kvoptions.sty', 'hyperref.sty']
 
 LATEX_WARNINGS = ENV_WARNINGS + """\
@@ -66,13 +67,16 @@ def compile_latex_document(app):
     with cd(app.outdir):
         try:
             ensuredir(app.config.latex_engine)
+            # keep a copy of latex file for this engine in case test fails
+            copyfile('SphinxTests.tex',
+                     app.config.latex_engine + '/SphinxTests.tex')
             p = Popen([app.config.latex_engine,
                        '--interaction=nonstopmode',
                        '-output-directory=%s' % app.config.latex_engine,
                        'SphinxTests.tex'],
                       stdout=PIPE, stderr=PIPE)
         except OSError:  # most likely the latex executable was not found
-            raise SkipTest
+            raise pytest.skip.Exception
         else:
             stdout, stderr = p.communicate()
             if p.returncode != 0:
@@ -82,14 +86,23 @@ def compile_latex_document(app):
                     app.config.latex_engine, p.returncode)
 
 
-def skip_if_stylefiles_notfound(testfunc):
-    if kpsetest(*STYLEFILES) is False:
-        msg = 'not running latex, the required styles do not seem to be installed'
-        return skip_if(True, msg)(testfunc)
+def skip_if_requested(testfunc):
+    if 'SKIP_LATEX_BUILD' in os.environ:
+        msg = 'Skip LaTeX builds because SKIP_LATEX_BUILD is set'
+        return pytest.mark.skipif(True, reason=msg)(testfunc)
     else:
         return testfunc
 
 
+def skip_if_stylefiles_notfound(testfunc):
+    if kpsetest(*STYLEFILES) is False:
+        msg = 'not running latex, the required styles do not seem to be installed'
+        return pytest.mark.skipif(True, reason=msg)(testfunc)
+    else:
+        return testfunc
+
+
+@skip_if_requested
 @skip_if_stylefiles_notfound
 @pytest.mark.parametrize(
     "engine,docclass",
@@ -116,24 +129,24 @@ def test_writer(app, status, warning):
 
     assert ('\\begin{sphinxfigure-in-table}\n\\centering\n\\capstart\n'
             '\\noindent\\sphinxincludegraphics{{img}.png}\n'
-            '\\sphinxfigcaption{figure in table}\\label{\\detokenize{markup:id7}}'
+            '\\sphinxfigcaption{figure in table}\\label{\\detokenize{markup:id8}}'
             '\\end{sphinxfigure-in-table}\\relax' in result)
 
     assert ('\\begin{wrapfigure}{r}{0pt}\n\\centering\n'
             '\\noindent\\sphinxincludegraphics{{rimg}.png}\n'
-            '\\caption{figure with align option}\\label{\\detokenize{markup:id8}}'
+            '\\caption{figure with align option}\\label{\\detokenize{markup:id9}}'
             '\\end{wrapfigure}' in result)
 
     assert ('\\begin{wrapfigure}{r}{0.500\\linewidth}\n\\centering\n'
             '\\noindent\\sphinxincludegraphics{{rimg}.png}\n'
             '\\caption{figure with align \\& figwidth option}'
-            '\\label{\\detokenize{markup:id9}}'
+            '\\label{\\detokenize{markup:id10}}'
             '\\end{wrapfigure}' in result)
 
     assert ('\\begin{wrapfigure}{r}{3cm}\n\\centering\n'
             '\\noindent\\sphinxincludegraphics[width=3cm]{{rimg}.png}\n'
             '\\caption{figure with align \\& width option}'
-            '\\label{\\detokenize{markup:id10}}'
+            '\\label{\\detokenize{markup:id11}}'
             '\\end{wrapfigure}' in result)
 
 
@@ -141,7 +154,7 @@ def test_writer(app, status, warning):
 def test_latex_warnings(app, status, warning):
     app.builder.build_all()
 
-    warnings = strip_escseq(warning.getvalue().replace(os.sep, '/'))
+    warnings = strip_escseq(re.sub(re.escape(os.sep) + '{1,2}', '/', warning.getvalue()))
     warnings_exp = LATEX_WARNINGS % {
         'root': re.escape(app.srcdir.replace(os.sep, '/'))}
     assert re.match(warnings_exp + '$', warnings), \
@@ -151,13 +164,15 @@ def test_latex_warnings(app, status, warning):
 
 
 @pytest.mark.sphinx('latex', testroot='basic')
-def test_latex_title(app, status, warning):
+def test_latex_basic(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'test.tex').text(encoding='utf8')
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert '\\title{The basic Sphinx documentation for testing}' in result
+    assert r'\title{The basic Sphinx documentation for testing}' in result
+    assert r'\release{}' in result
+    assert r'\renewcommand{\releasename}{}' in result
 
 
 @pytest.mark.sphinx('latex', testroot='latex-title')
@@ -170,8 +185,20 @@ def test_latex_title_after_admonitions(app, status, warning):
     assert '\\title{test-latex-title}' in result
 
 
+@pytest.mark.sphinx('latex', testroot='basic',
+                    confoverrides={'release': '1.0'})
+def test_latex_release(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'test.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert r'\release{1.0}' in result
+    assert r'\renewcommand{\releasename}{Release}' in result
+
+
 @pytest.mark.sphinx('latex', testroot='numfig',
-          confoverrides={'numfig': True})
+                    confoverrides={'numfig': True})
 def test_numref(app, status, warning):
     app.builder.build_all()
     result = (app.outdir / 'Python.tex').text(encoding='utf8')
@@ -296,19 +323,19 @@ def test_numref_with_language_ja(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert u'\\renewcommand{\\figurename}{\u56f3}' in result
-    assert '\\renewcommand{\\tablename}{TABLE}' in result
-    assert '\\renewcommand{\\literalblockname}{LIST}' in result
+    assert u'\\renewcommand{\\figurename}{\u56f3}' in result  # 図
+    assert u'\\renewcommand{\\tablename}{\u8868}' in result  # 表
+    assert u'\\renewcommand{\\literalblockname}{\u30ea\u30b9\u30c8}' in result  # リスト
     assert (u'\\hyperref[\\detokenize{index:fig1}]'
             u'{\u56f3 \\ref{\\detokenize{index:fig1}}}') in result
     assert ('\\hyperref[\\detokenize{baz:fig22}]'
             '{Figure\\ref{\\detokenize{baz:fig22}}}') in result
-    assert ('\\hyperref[\\detokenize{index:table-1}]'
-            '{TABLE \\ref{\\detokenize{index:table-1}}}') in result
+    assert (u'\\hyperref[\\detokenize{index:table-1}]'
+            u'{\u8868 \\ref{\\detokenize{index:table-1}}}') in result
     assert ('\\hyperref[\\detokenize{baz:table22}]'
             '{Table:\\ref{\\detokenize{baz:table22}}}') in result
-    assert ('\\hyperref[\\detokenize{index:code-1}]'
-            '{LIST \\ref{\\detokenize{index:code-1}}}') in result
+    assert (u'\\hyperref[\\detokenize{index:code-1}]'
+            u'{\u30ea\u30b9\u30c8 \\ref{\\detokenize{index:code-1}}}') in result
     assert ('\\hyperref[\\detokenize{baz:code22}]'
             '{Code-\\ref{\\detokenize{baz:code22}}}') in result
     assert (u'\\hyperref[\\detokenize{foo:foo}]'
@@ -321,6 +348,56 @@ def test_numref_with_language_ja(app, status, warning):
             '\\nameref{\\detokenize{foo:foo}}}') in result
 
 
+@pytest.mark.sphinx('latex', testroot='latex-numfig')
+def test_latex_obey_numfig_is_false(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage{sphinx}' in result
+
+
+@pytest.mark.sphinx(
+    'latex', testroot='latex-numfig',
+    confoverrides={'numfig': True, 'numfig_secnum_depth': 0})
+def test_latex_obey_numfig_secnum_depth_is_zero(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage[,nonumfigreset,mathnumfig]{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage[,nonumfigreset,mathnumfig]{sphinx}' in result
+
+
+@pytest.mark.sphinx(
+    'latex', testroot='latex-numfig',
+    confoverrides={'numfig': True, 'numfig_secnum_depth': 2})
+def test_latex_obey_numfig_secnum_depth_is_two(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=2,mathnumfig]{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=3,mathnumfig]{sphinx}' in result
+
+
+@pytest.mark.sphinx(
+    'latex', testroot='latex-numfig',
+    confoverrides={'numfig': True, 'math_numfig': False})
+def test_latex_obey_numfig_but_math_numfig_false(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'SphinxManual.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=1]{sphinx}' in result
+
+    result = (app.outdir / 'SphinxHowTo.tex').text(encoding='utf8')
+    assert '\\usepackage[,numfigreset=2]{sphinx}' in result
+
+
 @pytest.mark.sphinx('latex')
 def test_latex_add_latex_package(app, status, warning):
     app.add_latex_package('foo')
@@ -478,13 +555,17 @@ def test_footnote(app, status, warning):
     assert ('\\bibitem[bar]{\\detokenize{bar}}'
             '{\\phantomsection\\label{\\detokenize{footnote:bar}} '
             '\ncite\n}') in result
-    assert '\\caption{Table caption \\sphinxfootnotemark[4]' in result
-    assert 'name \\sphinxfootnotemark[5]' in result
-    assert ('\\end{threeparttable}\n\n%\n'
-            '\\begin{footnotetext}[4]\sphinxAtStartFootnote\n'
-            'footnotes in table caption\n%\n\\end{footnotetext}%\n'
-            '\\begin{footnotetext}[5]\sphinxAtStartFootnote\n'
-            'footnotes in table\n%\n\\end{footnotetext}') in result
+    assert '\\sphinxcaption{Table caption \\sphinxfootnotemark[4]' in result
+    assert ('\\hline%\n\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
+            'footnote in table caption\n%\n\\end{footnotetext}\\ignorespaces %\n'
+            '\\begin{footnotetext}[5]\\sphinxAtStartFootnote\n'
+            'footnote in table header\n%\n\\end{footnotetext}\\ignorespaces \n'
+            'VIDIOC\\_CROPCAP\n&\n') in result
+    assert ('Information about VIDIOC\\_CROPCAP %\n'
+            '\\begin{footnote}[6]\\sphinxAtStartFootnote\n'
+            'footnote in table not in header\n%\n\\end{footnote}\n\\\\\n\\hline\n'
+            '\\end{tabulary}\n'
+            '\\par\n\\sphinxattableend\\end{savenotes}\n') in result
 
 
 @pytest.mark.sphinx('latex', testroot='footnotes')
@@ -499,28 +580,32 @@ def test_reference_in_caption_and_codeblock_in_footnote(app, status, warning):
             '{\\hyperref[\\detokenize{index:authoryear}]'
             '{\\sphinxcrossref{{[}AuthorYear{]}}}}.}' in result)
     assert '\\chapter{The section with a reference to {[}AuthorYear{]}}' in result
-    assert '\\caption{The table title with a reference to {[}AuthorYear{]}}' in result
+    assert ('\\sphinxcaption{The table title with a reference'
+            ' to {[}AuthorYear{]}}' in result)
     assert '\\paragraph{The rubric title with a reference to {[}AuthorYear{]}}' in result
     assert ('\\chapter{The section with a reference to \\sphinxfootnotemark[4]}\n'
             '\\label{\\detokenize{index:the-section-with-a-reference-to}}'
             '%\n\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
             'Footnote in section\n%\n\\end{footnotetext}') in result
     assert ('\\caption{This is the figure caption with a footnote to '
-            '\\sphinxfootnotemark[6].}\label{\\detokenize{index:id27}}\end{figure}\n'
+            '\\sphinxfootnotemark[6].}\\label{\\detokenize{index:id27}}\\end{figure}\n'
             '%\n\\begin{footnotetext}[6]\\sphinxAtStartFootnote\n'
             'Footnote in caption\n%\n\\end{footnotetext}')in result
-    assert ('\\caption{footnote \\sphinxfootnotemark[7] '
-            'in caption of normal table}\\label{\\detokenize{index:id28}}') in result
+    assert ('\\sphinxcaption{footnote \\sphinxfootnotemark[7] in '
+            'caption of normal table}\\label{\\detokenize{index:id28}}') in result
     assert ('\\caption{footnote \\sphinxfootnotemark[8] '
-            'in caption \sphinxfootnotemark[9] of longtable}') in result
-    assert ('\end{longtable}\n\n%\n\\begin{footnotetext}[8]'
-            '\sphinxAtStartFootnote\n'
-            'Foot note in longtable\n%\n\\end{footnotetext}' in result)
+            'in caption \\sphinxfootnotemark[9] of longtable\\strut}') in result
+    assert ('\\endlastfoot\n%\n\\begin{footnotetext}[8]\\sphinxAtStartFootnote\n'
+            'Foot note in longtable\n%\n\\end{footnotetext}\\ignorespaces %\n'
+            '\\begin{footnotetext}[9]\\sphinxAtStartFootnote\n'
+            'Second footnote in caption of longtable\n') in result
     assert ('This is a reference to the code-block in the footnote:\n'
-            '{\hyperref[\\detokenize{index:codeblockinfootnote}]'
+            '{\\hyperref[\\detokenize{index:codeblockinfootnote}]'
             '{\\sphinxcrossref{\\DUrole{std,std-ref}{I am in a footnote}}}}') in result
-    assert ('&\nThis is one more footnote with some code in it '
-            '\\sphinxfootnotemark[10].\n\\\\') in result
+    assert ('&\nThis is one more footnote with some code in it %\n'
+            '\\begin{footnote}[10]\\sphinxAtStartFootnote\n'
+            'Third footnote in longtable\n') in result
+    assert ('\\end{sphinxVerbatim}\n%\n\\end{footnote}.\n') in result
     assert '\\begin{sphinxVerbatim}[commandchars=\\\\\\{\\}]' in result
 
 
@@ -548,23 +633,24 @@ def test_latex_show_urls_is_inline(app, status, warning):
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to}]'
             '{\\sphinxcrossref{The section with a reference to }}}' in result)
     assert ('First footnote: %\n\\begin{footnote}[2]\\sphinxAtStartFootnote\n'
-           'First\n%\n\\end{footnote}') in result
+            'First\n%\n\\end{footnote}') in result
     assert ('Second footnote: %\n\\begin{footnote}[1]\\sphinxAtStartFootnote\n'
-           'Second\n%\n\\end{footnote}') in result
-    assert '\\href{http://sphinx-doc.org/}{Sphinx} (http://sphinx-doc.org/)' in result
+            'Second\n%\n\\end{footnote}') in result
+    assert '\\sphinxhref{http://sphinx-doc.org/}{Sphinx} (http://sphinx-doc.org/)' in result
     assert ('Third footnote: %\n\\begin{footnote}[3]\\sphinxAtStartFootnote\n'
-           'Third\n%\n\\end{footnote}') in result
-    assert ('\\href{http://sphinx-doc.org/~test/}{URL including tilde} '
+            'Third\n%\n\\end{footnote}') in result
+    assert ('\\sphinxhref{http://sphinx-doc.org/~test/}{URL including tilde} '
             '(http://sphinx-doc.org/\\textasciitilde{}test/)') in result
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term} '
+    assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{URL in term} '
             '(http://sphinx-doc.org/)}] \\leavevmode\nDescription' in result)
     assert ('\\item[{Footnote in term \\sphinxfootnotemark[5]}] '
             '\\leavevmode%\n\\begin{footnotetext}[5]\\sphinxAtStartFootnote\n'
-            'Footnote in term\n%\n\\end{footnotetext}\nDescription') in result
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist} '
+            'Footnote in term\n%\n\\end{footnotetext}\\ignorespaces \n'
+            'Description') in result
+    assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{Term in deflist} '
             '(http://sphinx-doc.org/)}] \\leavevmode\nDescription') in result
-    assert '\\url{https://github.com/sphinx-doc/sphinx}\n' in result
-    assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
+    assert '\\sphinxurl{https://github.com/sphinx-doc/sphinx}\n' in result
+    assert ('\\sphinxhref{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}') in result
 
 
@@ -591,32 +677,33 @@ def test_latex_show_urls_is_footnote(app, status, warning):
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to}]'
             '{\\sphinxcrossref{The section with a reference to }}}') in result
     assert ('First footnote: %\n\\begin{footnote}[3]\\sphinxAtStartFootnote\n'
-           'First\n%\n\\end{footnote}') in result
+            'First\n%\n\\end{footnote}') in result
     assert ('Second footnote: %\n\\begin{footnote}[1]\\sphinxAtStartFootnote\n'
-           'Second\n%\n\\end{footnote}') in result
-    assert ('\\href{http://sphinx-doc.org/}{Sphinx}'
+            'Second\n%\n\\end{footnote}') in result
+    assert ('\\sphinxhref{http://sphinx-doc.org/}{Sphinx}'
             '%\n\\begin{footnote}[4]\\sphinxAtStartFootnote\n'
-            '\\nolinkurl{http://sphinx-doc.org/}\n%\n\\end{footnote}') in result
+            '\\sphinxnolinkurl{http://sphinx-doc.org/}\n%\n\\end{footnote}') in result
     assert ('Third footnote: %\n\\begin{footnote}[6]\\sphinxAtStartFootnote\n'
             'Third\n%\n\\end{footnote}') in result
-    assert ('\\href{http://sphinx-doc.org/~test/}{URL including tilde}'
+    assert ('\\sphinxhref{http://sphinx-doc.org/~test/}{URL including tilde}'
             '%\n\\begin{footnote}[5]\\sphinxAtStartFootnote\n'
-            '\\nolinkurl{http://sphinx-doc.org/~test/}\n%\n\\end{footnote}') in result
-    assert ('\\item[{\\href{http://sphinx-doc.org/}'
+            '\\sphinxnolinkurl{http://sphinx-doc.org/~test/}\n%\n\\end{footnote}') in result
+    assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}'
             '{URL in term}\\sphinxfootnotemark[8]}] '
             '\\leavevmode%\n\\begin{footnotetext}[8]\\sphinxAtStartFootnote\n'
-            '\\nolinkurl{http://sphinx-doc.org/}\n%\n'
-            '\\end{footnotetext}\nDescription') in result
+            '\\sphinxnolinkurl{http://sphinx-doc.org/}\n%\n'
+            '\\end{footnotetext}\\ignorespaces \nDescription') in result
     assert ('\\item[{Footnote in term \\sphinxfootnotemark[10]}] '
             '\\leavevmode%\n\\begin{footnotetext}[10]\\sphinxAtStartFootnote\n'
-            'Footnote in term\n%\n\\end{footnotetext}\nDescription') in result
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}'
+            'Footnote in term\n%\n\\end{footnotetext}\\ignorespaces \n'
+            'Description') in result
+    assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{Term in deflist}'
             '\\sphinxfootnotemark[9]}] '
             '\\leavevmode%\n\\begin{footnotetext}[9]\\sphinxAtStartFootnote\n'
-            '\\nolinkurl{http://sphinx-doc.org/}\n%\n'
-            '\\end{footnotetext}\nDescription') in result
-    assert  '\\url{https://github.com/sphinx-doc/sphinx}\n' in result
-    assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
+            '\\sphinxnolinkurl{http://sphinx-doc.org/}\n%\n'
+            '\\end{footnotetext}\\ignorespaces \nDescription') in result
+    assert ('\\sphinxurl{https://github.com/sphinx-doc/sphinx}\n' in result)
+    assert ('\\sphinxhref{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}\n') in result
 
 
@@ -643,22 +730,23 @@ def test_latex_show_urls_is_no(app, status, warning):
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to}]'
             '{\\sphinxcrossref{The section with a reference to }}}' in result)
     assert ('First footnote: %\n\\begin{footnote}[2]\\sphinxAtStartFootnote\n'
-           'First\n%\n\\end{footnote}') in result
+            'First\n%\n\\end{footnote}') in result
     assert ('Second footnote: %\n\\begin{footnote}[1]\\sphinxAtStartFootnote\n'
-           'Second\n%\n\\end{footnote}') in result
-    assert '\\href{http://sphinx-doc.org/}{Sphinx}' in result
+            'Second\n%\n\\end{footnote}') in result
+    assert '\\sphinxhref{http://sphinx-doc.org/}{Sphinx}' in result
     assert ('Third footnote: %\n\\begin{footnote}[3]\\sphinxAtStartFootnote\n'
-           'Third\n%\n\\end{footnote}') in result
-    assert '\\href{http://sphinx-doc.org/~test/}{URL including tilde}' in result
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}}] '
+            'Third\n%\n\\end{footnote}') in result
+    assert '\\sphinxhref{http://sphinx-doc.org/~test/}{URL including tilde}' in result
+    assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{URL in term}}] '
             '\\leavevmode\nDescription') in result
     assert ('\\item[{Footnote in term \\sphinxfootnotemark[5]}] '
             '\\leavevmode%\n\\begin{footnotetext}[5]\\sphinxAtStartFootnote\n'
-            'Footnote in term\n%\n\\end{footnotetext}\nDescription') in result
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}}] '
+            'Footnote in term\n%\n\\end{footnotetext}\\ignorespaces \n'
+            'Description') in result
+    assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{Term in deflist}}] '
             '\\leavevmode\nDescription') in result
-    assert ('\\url{https://github.com/sphinx-doc/sphinx}\n' in result)
-    assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
+    assert ('\\sphinxurl{https://github.com/sphinx-doc/sphinx}\n' in result)
+    assert ('\\sphinxhref{mailto:sphinx-dev@googlegroups.com}'
             '{sphinx-dev@googlegroups.com}\n') in result
 
 
@@ -678,7 +766,8 @@ def test_image_in_section(app, status, warning):
     assert ('\\chapter{Another section}' in result)
 
 
-@pytest.mark.sphinx('latex', confoverrides={'latex_logo': 'notfound.jpg'})
+@pytest.mark.sphinx('latex', testroot='basic',
+                    confoverrides={'latex_logo': 'notfound.jpg'})
 def test_latex_logo_if_not_found(app, status, warning):
     try:
         app.builder.build_all()
@@ -687,19 +776,16 @@ def test_latex_logo_if_not_found(app, status, warning):
         assert isinstance(exc, SphinxError)
 
 
-@pytest.mark.sphinx('latex', testroot='toctree-maxdepth',
-          confoverrides={'latex_documents': [
-              ('index', 'SphinxTests.tex', 'Sphinx Tests Documentation',
-               'Georg Brandl', 'manual'),
-          ]})
+@pytest.mark.sphinx('latex', testroot='toctree-maxdepth')
 def test_toctree_maxdepth_manual(app, status, warning):
     app.builder.build_all()
-    result = (app.outdir / 'SphinxTests.tex').text(encoding='utf8')
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
     assert '\\setcounter{tocdepth}{1}' in result
     assert '\\setcounter{secnumdepth}' not in result
+    assert '\\chapter{Foo}' in result
 
 
 @pytest.mark.sphinx(
@@ -716,6 +802,7 @@ def test_toctree_maxdepth_howto(app, status, warning):
     print(warning.getvalue())
     assert '\\setcounter{tocdepth}{2}' in result
     assert '\\setcounter{secnumdepth}' not in result
+    assert '\\section{Foo}' in result
 
 
 @pytest.mark.sphinx(
@@ -729,6 +816,7 @@ def test_toctree_not_found(app, status, warning):
     print(warning.getvalue())
     assert '\\setcounter{tocdepth}' not in result
     assert '\\setcounter{secnumdepth}' not in result
+    assert '\\chapter{Foo A}' in result
 
 
 @pytest.mark.sphinx(
@@ -779,6 +867,26 @@ def test_latex_toplevel_sectioning_is_part(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert '\\part{Foo}' in result
+    assert '\\chapter{Foo A}' in result
+    assert '\\chapter{Foo B}' in result
+
+
+@pytest.mark.sphinx(
+    'latex', testroot='toctree-maxdepth',
+    confoverrides={'latex_toplevel_sectioning': 'part',
+                   'latex_documents': [
+                       ('index', 'Python.tex', 'Sphinx Tests Documentation',
+                        'Georg Brandl', 'howto')
+                   ]})
+def test_latex_toplevel_sectioning_is_part_with_howto(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\part{Foo}' in result
+    assert '\\section{Foo A}' in result
+    assert '\\section{Foo B}' in result
 
 
 @pytest.mark.sphinx(
@@ -793,6 +901,22 @@ def test_latex_toplevel_sectioning_is_chapter(app, status, warning):
     assert '\\chapter{Foo}' in result
 
 
+@pytest.mark.sphinx(
+    'latex', testroot='toctree-maxdepth',
+    confoverrides={'latex_toplevel_sectioning': 'chapter',
+                   'latex_documents': [
+                       ('index', 'Python.tex', 'Sphinx Tests Documentation',
+                        'Georg Brandl', 'howto')
+                   ]})
+def test_latex_toplevel_sectioning_is_chapter_with_howto(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\section{Foo}' in result
+
+
 @pytest.mark.sphinx(
     'latex', testroot='toctree-maxdepth',
     confoverrides={'latex_toplevel_sectioning': 'section'})
@@ -804,6 +928,7 @@ def test_latex_toplevel_sectioning_is_section(app, status, warning):
     print(warning.getvalue())
     assert '\\section{Foo}' in result
 
+
 @skip_if_stylefiles_notfound
 @pytest.mark.sphinx('latex', testroot='maxlistdepth')
 def test_maxlistdepth_at_ten(app, status, warning):
@@ -813,3 +938,218 @@ def test_maxlistdepth_at_ten(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     compile_latex_document(app)
+
+
+@pytest.mark.skipif(docutils.__version_info__ < (0, 13),
+                    reason='docutils-0.13 or above is required')
+@pytest.mark.sphinx('latex', testroot='latex-table')
+@pytest.mark.test_params(shared_result='latex-table')
+def test_latex_table_tabulars(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'test.tex').text(encoding='utf8')
+    tables = {}
+    for chap in re.split(r'\\(?:section|chapter){', result)[1:]:
+        sectname, content = chap.split('}', 1)
+        tables[sectname] = content.strip()
+
+    def get_expected(name):
+        return (app.srcdir / 'expects' / (name + '.tex')).text().strip()
+
+    # simple_table
+    actual = tables['simple table']
+    expected = get_expected('simple_table')
+    assert actual == expected
+
+    # table having :widths: option
+    actual = tables['table having :widths: option']
+    expected = get_expected('table_having_widths')
+    assert actual == expected
+
+    # table having :align: option (tabulary)
+    actual = tables['table having :align: option (tabulary)']
+    expected = get_expected('tabulary_having_widths')
+    assert actual == expected
+
+    # table having :align: option (tabular)
+    actual = tables['table having :align: option (tabular)']
+    expected = get_expected('tabular_having_widths')
+    assert actual == expected
+
+    # table with tabularcolumn
+    actual = tables['table with tabularcolumn']
+    expected = get_expected('tabularcolumn')
+    assert actual == expected
+
+    # table with cell in first column having three paragraphs
+    actual = tables['table with cell in first column having three paragraphs']
+    expected = get_expected('table_having_threeparagraphs_cell_in_first_col')
+    assert actual == expected
+
+    # table having caption
+    actual = tables['table having caption']
+    expected = get_expected('table_having_caption')
+    assert actual == expected
+
+    # table having verbatim
+    actual = tables['table having verbatim']
+    expected = get_expected('table_having_verbatim')
+    assert actual == expected
+
+    # table having problematic cell
+    actual = tables['table having problematic cell']
+    expected = get_expected('table_having_problematic_cell')
+    assert actual == expected
+
+    # table having both :widths: and problematic cell
+    actual = tables['table having both :widths: and problematic cell']
+    expected = get_expected('table_having_widths_and_problematic_cell')
+    assert actual == expected
+
+    # table having both stub columns and problematic cell
+    actual = tables['table having both stub columns and problematic cell']
+    expected = get_expected('table_having_stub_columns_and_problematic_cell')
+    assert actual == expected
+
+
+@pytest.mark.skipif(docutils.__version_info__ < (0, 13),
+                    reason='docutils-0.13 or above is required')
+@pytest.mark.sphinx('latex', testroot='latex-table')
+@pytest.mark.test_params(shared_result='latex-table')
+def test_latex_table_longtable(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'test.tex').text(encoding='utf8')
+    tables = {}
+    for chap in re.split(r'\\(?:section|chapter){', result)[1:]:
+        sectname, content = chap.split('}', 1)
+        tables[sectname] = content.strip()
+
+    def get_expected(name):
+        return (app.srcdir / 'expects' / (name + '.tex')).text().strip()
+
+    # longtable
+    actual = tables['longtable']
+    expected = get_expected('longtable')
+    assert actual == expected
+
+    # longtable having :widths: option
+    actual = tables['longtable having :widths: option']
+    expected = get_expected('longtable_having_widths')
+    assert actual == expected
+
+    # longtable having :align: option
+    actual = tables['longtable having :align: option']
+    expected = get_expected('longtable_having_align')
+    assert actual == expected
+
+    # longtable with tabularcolumn
+    actual = tables['longtable with tabularcolumn']
+    expected = get_expected('longtable_with_tabularcolumn')
+    assert actual == expected
+
+    # longtable having caption
+    actual = tables['longtable having caption']
+    expected = get_expected('longtable_having_caption')
+    assert actual == expected
+
+    # longtable having verbatim
+    actual = tables['longtable having verbatim']
+    expected = get_expected('longtable_having_verbatim')
+    assert actual == expected
+
+    # longtable having problematic cell
+    actual = tables['longtable having problematic cell']
+    expected = get_expected('longtable_having_problematic_cell')
+    assert actual == expected
+
+    # longtable having both :widths: and problematic cell
+    actual = tables['longtable having both :widths: and problematic cell']
+    expected = get_expected('longtable_having_widths_and_problematic_cell')
+    assert actual == expected
+
+    # longtable having both stub columns and problematic cell
+    actual = tables['longtable having both stub columns and problematic cell']
+    expected = get_expected('longtable_having_stub_columns_and_problematic_cell')
+    assert actual == expected
+
+
+@pytest.mark.skipif(docutils.__version_info__ < (0, 13),
+                    reason='docutils-0.13 or above is required')
+@pytest.mark.sphinx('latex', testroot='latex-table')
+@pytest.mark.test_params(shared_result='latex-table')
+def test_latex_table_complex_tables(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'test.tex').text(encoding='utf8')
+    tables = {}
+    for chap in re.split(r'\\(?:section|renewcommand){', result)[1:]:
+        sectname, content = chap.split('}', 1)
+        tables[sectname] = content.strip()
+
+    def get_expected(name):
+        return (app.srcdir / 'expects' / (name + '.tex')).text().strip()
+
+    # grid table
+    actual = tables['grid table']
+    expected = get_expected('gridtable')
+    assert actual == expected
+
+    # complex spanning cell
+    actual = tables['complex spanning cell']
+    expected = get_expected('complex_spanning_cell')
+    assert actual == expected
+
+
+@pytest.mark.sphinx('latex', testroot='directives-raw')
+def test_latex_raw_directive(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+
+    # standard case
+    assert 'standalone raw directive (HTML)' not in result
+    assert ('\\label{\\detokenize{index:id1}}\n'
+            'standalone raw directive (LaTeX)' in result)
+
+    # with substitution
+    assert 'HTML: abc  ghi' in result
+    assert 'LaTeX: abc def ghi' in result
+
+
+@pytest.mark.sphinx('latex', testroot='images')
+def test_latex_remote_images(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    assert '\\sphinxincludegraphics{{python-logo}.png}' in result
+    assert (app.outdir / 'python-logo.png').exists()
+    assert '\\sphinxincludegraphics{{NOT_EXIST}.PNG}' not in result
+    assert ('WARNING: Could not fetch remote image: '
+            'http://example.com/NOT_EXIST.PNG [404]' in warning.getvalue())
+
+
+@pytest.mark.sphinx('latex', testroot='latex-index')
+def test_latex_index(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    assert 'A \\index{famous}famous \\index{equation}equation:\n' in result
+    assert '\n\\index{Einstein}\\index{relativity}\\ignorespaces \nand' in result
+
+
+@pytest.mark.sphinx('latex', testroot='image-in-parsed-literal')
+def test_latex_image_in_parsed_literal(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    assert ('{\\sphinxunactivateextrasandspace \\raisebox{-0.5\\height}'
+            '{\\scalebox{2.000000}{\\sphinxincludegraphics[height=1cm]{{pic}.png}}}'
+            '}AFTER') in result
+
+
+@pytest.mark.sphinx('latex', testroot='nested-enumerated-list')
+def test_latex_nested_enumerated_list(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'test.tex').text(encoding='utf8')
+    assert r'\setcounter{enumi}{4}' in result
+    assert r'\setcounter{enumii}{3}' in result
+    assert r'\setcounter{enumiii}{9}' in result
+    assert r'\setcounter{enumii}{2}' in result
