@@ -5,19 +5,19 @@
 
     Test the search index builder.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from collections import namedtuple
 
-from six import BytesIO
+import pytest
 from docutils import frontend, utils
 from docutils.parsers import rst
+from six import BytesIO
 
 from sphinx.search import IndexBuilder
 from sphinx.util import jsdump
-import pytest
 
 DummyEnvironment = namedtuple('DummyEnvironment', ['version', 'domains'])
 
@@ -228,3 +228,15 @@ def test_IndexBuilder():
     }
     assert index._objtypes == {('dummy', 'objtype'): 0}
     assert index._objnames == {0: ('dummy', 'objtype', 'objtype')}
+
+
+def test_IndexBuilder_lookup():
+    env = DummyEnvironment('1.0', {})
+
+    # zh
+    index = IndexBuilder(env, 'zh', {}, None)
+    assert index.lang.lang == 'zh'
+
+    # zh_CN
+    index = IndexBuilder(env, 'zh_CN', {}, None)
+    assert index.lang.lang == 'zh'
